require 'version'
require 'httpi'

module Acoustid
  
  module API
    
    is_versioned
    
    class << self
      
      def lookup(options={})
        options = parse_options(options)
        
        require 'acoustid/api/request/lookup'
        
        request = Request::Lookup.new( options[:params] )
        
        send( options[:method], request, options[:adapter] )
      end
      
      def get(request, http_adapter=nil)
        raise TypeError unless request.is_a?(Request::Base)
        
        http_request = HTTPI::Request.new( url: request.uri, query: request.params )
        
        HTTPI.get(http_request, http_adapter)
      end
      
      def post(request, adapter=nil)
        raise TypeError unless request.is_a?(Request::Base)
        
        http_request = HTTPI::Request.new( url: request.uri, body: request.params )
        
        HTTPI.post(http_request, adapter)
      end
      
      protected
      
      def parse_options(options={})
        raise TypeError, 'options must respond to :to_hash or :to_h' unless options.respond_to?(:to_hash) || options.respond_to?(:to_h)
        options = options.to_hash rescue options.to_h
        
        options = { method: :post }.merge(options)
        
        raise TypeError, 'option :method must respond_to :to_sym' unless options[:method].respond_to?(:to_sym)
        options[:method] = options[:method].to_s.strip.to_sym
        raise TypeError, 'option :method must be either :get or :post' unless [:get, :post].include?( options[:method] )
        
        raise TypeError, 'option :params must respond to :to_hash or :to_h' unless options[:params].respond_to?(:to_hash) || options[:params].respond_to?(:to_h)
        options[:params] = options[:params].to_hash rescue options[:params].to_h
        
        raise TypeError, 'option :adapter must respond_to :to_sym or be nil' unless options[:adapter].nil? || options[:adapter].respond_to?(:to_sym)
        options[:adapter] = options[:adapter].to_s.strip.to_sym unless options[:adapter].nil?
        
        options
      end
      
    end
    
  end
  
end

# request = Acoustid::API.lookup(params: { client: 'omgomg' })

__END__

Lookup by fingerprint
http://api.acoustid.org/v2/lookup

Name  Required  Values  Description
format  no  json, jsonp, xml  response format
jsoncallback  no  jsonAcoustidApi  
client  yes  8XaBELgH  application's API key
duration  yes  641  duration of the whole audio file in seconds
fingerprint  yes  AQABz0qUkZ...  audio fingerprint data
meta  no  recordings, recordingids, releases, releaseids, releasegroups, releasegroupids, tracks, compress, usermeta, sources  returned metadata

Lookup by track ID
http://api.acoustid.org/v2/lookup

Name  Required  Values  Description
format  no  json, jsonp, xml  response format
jsoncallback  no  jsonAcoustidApi  JSONP callback, only applicable if you select the jsonp format
client  yes  8XaBELgH  application's API key
trackid  yes  9ff43b6a-4f16-427c-93c2-92307ca505e0  track id (UUID)
meta  no  recordings, recordingids, releases, releaseids, releasegroups, releasegroupids, tracks, compress, usermeta, sources  returned metadata

Submit
http://api.acoustid.org/v2/submit

Name  Required  Values  Description
format  no  json, xml  response format
client  yes  8XaBELgH  application's API key
clientversion  no  1.0  application's version
wait  no  1  wait up to N seconds for the submission(s) to import
user  yes  9Hg1oNwY  users's API key
duration.#  yes  641  duration of the whole audio file in seconds
fingerprint.#  yes  AQABz0qUkZ...  audio fingerprint data
bitrate.#  no  192  bitrate of the audio file
fileformat.#  no  MP3, M4A, ...  file format of the audio file
mbid.#  no  4e0d8649-1f89-44f3-91af-4c0dbee81f28  corresponding MusicBrainz track ID
track.#  no  High Hopes  track title
artist.#  no  Pink Floyd  track artist
album.#  no  The Division Bell  album title
albumartist.#  no  Pink Floyd  album artist
year.#  no  1994  album release year
trackno.#  no  11  track number
discno.#  no  1  disc number

Get submission status (TODO: MIGHT HAVE TYPE IN URL!)
http://api.acoustid.org/v2/submision_status

Name  Required  Values  Description
format  no  json, xml  response format
client  yes  8XaBELgH  application's API key
clientversion  no  1.0  application's version
id  yes  123456789  submission ID, can be used multiple times


List AcoustIDs by MBID
http://api.acoustid.org/v2/track/list_by_mbid

Name  Required  Values  Description
format  no  json, jsonp, xml  response format
jsoncallback  no  jsonAcoustidApi  JSONP callback, only applicable if you select the jsonp format
mbid  yes  4e0d8649-1f89-44f3-91af-4c0dbee81f28  corresponding MusicBrainz recording ID (can be sent multiple times)
batch  no  0, 1  use if you want to lookup multiple MBIDs